﻿// (C) Copyright 2002-2009 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;

// This line is not mandatory, but improves loading performances
[assembly: CommandClass(typeof(Autodesk.AutoCAD.ThreeDimensions.MyCommands))]

namespace Autodesk.AutoCAD.ThreeDimensions
{
  // This class is instantiated by AutoCAD for each document when
  // a command is called by the user the first time in the context
  // of a given document. In other words, non static data in this class
  // is implicitly per-document!
  public partial class MyCommands
  {
    // The CommandMethod attribute can be applied to any public  member 
    // function of any public class.
    // The function should take no arguments and return nothing.
    // If the method is an instance member then the enclosing class is 
    // instantiated for each document. If the member is a static member then
    // the enclosing class is NOT instantiated.
    //
    // NOTE: CommandMethod has overloads where you can provide helpid and
    // context menu.
    [CommandMethod("extrudeGeometry")]
    public void extrudeGeometry()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // pick a point to place this
      PromptPointOptions opts = new PromptPointOptions("Pick insertion point");
      PromptPointResult res = ed.GetPoint(opts);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // extract the pick point into a 2d point
        Point2d pnt = new Point2d(res.Value.X, res.Value.Y);

        // create somethings to loft
        using (Polyline star1 = Utils.CreateStarOutline(pnt, 0))
        {
          // add the result to the current space
          using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
            curSpace.AppendEntity(star1);
        }
      }
    }
    [CommandMethod("myExtrude")]
    public void myExtrude()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // select an object to extrude
      PromptEntityOptions opts = new PromptEntityOptions("Select Curve to Extrude");
      opts.SetRejectMessage("\nEntity must be a curve entity type like Polyline, Arc, Line, Circle, etc.");
      // set the entity types we allow
      opts.AddAllowedClass(typeof(Curve), false);
      PromptEntityResult res = ed.GetEntity(opts);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // open the selected item as a curve
        using (Curve curve = res.ObjectId.Open(OpenMode.ForRead) as Curve)
        {
          // now create an extrusion, we can actually set poly.Extrusion if we want
          using (ExtrudedSurface extrudedSurface = new ExtrudedSurface())
          {
            // create the SweepOptions for this extrude
            SweepOptions sweepOptions = new SweepOptions();
            // now the builder class for the sweep
            SweepOptionsBuilder builder = new SweepOptionsBuilder(sweepOptions);
            // we're not going to set anything in the builder, but it's there if we need it

            // create the height vector
            Vector3d heightVector = new Vector3d(0, 0, 40);

            // now create the extruded surface
            extrudedSurface.CreateExtrudedSurface(curve, heightVector, sweepOptions);

            // add the result to the current space
            using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
              curSpace.AppendEntity(extrudedSurface);
          }
        }
      }
    }
  }
}
