﻿// (C) Copyright 2002-2009 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;

// This line is not mandatory, but improves loading performances
[assembly: CommandClass(typeof(Autodesk.AutoCAD.ThreeDimensions.MyCommands))]

namespace Autodesk.AutoCAD.ThreeDimensions
{
  // This class is instantiated by AutoCAD for each document when
  // a command is called by the user the first time in the context
  // of a given document. In other words, non static data in this class
  // is implicitly per-document!
  public partial class MyCommands
  {
    // The CommandMethod attribute can be applied to any public  member 
    // function of any public class.
    // The function should take no arguments and return nothing.
    // If the method is an intance member then the enclosing class is 
    // intantiated for each document. If the member is a static member then
    // the enclosing class is NOT intantiated.
    //
    // NOTE: CommandMethod has overloads where you can provide helpid and
    // context menu.

    // sweep along a path
    [CommandMethod("sweepGeometry")]
    public void sweepGeometry()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // pick a point to place this
      PromptPointOptions opts = new PromptPointOptions("Pick insertion point");
      PromptPointResult res = ed.GetPoint(opts);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // extract the pick point into a 2d point
        Point2d pnt = new Point2d(res.Value.X, res.Value.Y);

        // create somethings to loft
        using (Polyline star1 = Utils.CreateStarOutline(pnt, 0))
        using (Circle circle = Utils.CreateCircle(res.Value, 2))
        {
          // rotate the circle about the yaxis 90 degrees
          Matrix3d rotate = new Matrix3d();
          rotate = Matrix3d.Rotation(Utils.pi / 2, Vector3d.YAxis, res.Value);
          circle.TransformBy(rotate);
          // add the result to the current space
          using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
          {
            curSpace.AppendEntity(star1);
            curSpace.AppendEntity(circle);
          }
        }
      }
    }

    [CommandMethod("mySweep")]
    public void sweepAlongPath()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // Ask the user to select a region to extrude
      PromptEntityOptions opts = new PromptEntityOptions("Select profile or curve to sweep");
      opts.SetRejectMessage("\nEntity must be a region, curve or planar surface.");

      opts.AddAllowedClass(typeof(Region), false);
      opts.AddAllowedClass(typeof(Curve), false);
      opts.AddAllowedClass(typeof(PlaneSurface), false);

      PromptEntityResult res = ed.GetEntity(opts);
      if (res.Status == PromptStatus.OK)
      {
        ObjectId regId = res.ObjectId;
        // Ask the user to select an extrusion path
        PromptEntityOptions opts2 = new PromptEntityOptions("Select path along which to sweep");
        opts2.SetRejectMessage("\nEntity must be a curve.");
        opts2.AddAllowedClass(typeof(Curve), false);
        res = ed.GetEntity(opts2);
        if (res.Status == PromptStatus.OK)
        {
          ObjectId splId = res.ObjectId;
          // Now let's create our swept surface
          using (Entity sweepEnt = regId.Open(OpenMode.ForRead) as Entity)
          {
            using (Curve pathEnt = splId.Open(OpenMode.ForRead) as Curve)
            {
              // We use a builder object to create
              // our SweepOptions
              SweepOptionsBuilder sob = new SweepOptionsBuilder();
              // Align the entity to sweep to the path
              sob.Align = SweepOptionsAlignOption.AlignSweepEntityToPath;
              // The base point is the start of the path
              sob.BasePoint = pathEnt.StartPoint;
              // The profile will rotate to follow the path
              sob.Bank = true;
              // Now generate the surface...
              using (SweptSurface ss = new SweptSurface())
              {
                ss.CreateSweptSurface(sweepEnt, pathEnt, sob.ToSweepOptions());
                // add the result to the current space
                using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
                  curSpace.AppendEntity(ss);
              }
            }
          }
        }
      }              
    }
  }
}
